#!/usr/bin/env python
import jppy
import numpy as np

r = jppy.PyJDAQTimesliceReader("/home/tgal/data/L0/KM3NeT_00000013_00000501.root")
i = 0
while r.has_next:
    r.retrieve_next_timeslice()
    j = 0
    while r.has_next_superframe:
        r.retrieve_next_superframe()
        n = r.number_of_hits
        channel_ids = np.zeros(n, dtype='i')
        dom_ids = np.zeros(n, dtype='i')
        times = np.zeros(n, dtype='i')
        tots = np.zeros(n, dtype='i')
        r.get_hits(channel_ids, dom_ids, times, tots)
        print(channel_ids)
        print(len(channel_ids))
        print(dom_ids)
        print(len(dom_ids))
        print(times)
        print(len(times))
        print(tots)
        print(len(tots))
        print("number of hits: {0}".format(n))
        raw_input()
        j += 1
    i += 1
print("{0} timeslices processed.".format(i))



from jppy.daqeventreader import PyJDAQEventReader
from jppy.daqsummaryslicereader import PyJDAQSummarysliceReader
from jppy.daqtimeslicereader import PyJDAQTimesliceReader
from jppy.fitreader import PyJFitReader



# distutils: language = c++
# distutils: sources = src/JDAQSummarysliceReader.cpp
# vim:set ts=4 sts=4 sw=4 et:

from libcpp cimport bool

import numpy as np
cimport numpy as np
cimport cython

np.import_array()


cdef extern from "JDAQSummarysliceReader.h" namespace "jppy":
    cdef cppclass JDAQSummarysliceReader:
        JDAQSummarysliceReader() except +
        JDAQSummarysliceReader(char* filename) except +

        bool hasNext()
        void retrieveNextSummaryslice()
        int getNumberOfFrames()
        int getRunNumber()
        int getDetectorID()
        int getFrameIndex()
        int getUTCSeconds()
        int getUTCNanoseconds()

        bool hasNextFrame()
        void retrieveNextFrame()
        int getModuleID()
        int getUDPNumberOfReceivedPackets()
        int getUDPMaximalSequenceNumber()
        bool hasUDPTrailer();
        bool testWhiteRabbitStatus();
        bool testHighRateVeto();
        bool testFIFOStatus();
        void getRates(double* rates)
        void getHRVs(int* hrvs)
        void getFIFOs(int* fifos)

cdef class PyJDAQSummarysliceReader:
    cdef JDAQSummarysliceReader c_reader

    def __cinit__(self, char* filename):
        self.c_reader = JDAQSummarysliceReader(filename)

    def retrieve_next_summaryslice(self):
        self.c_reader.retrieveNextSummaryslice()

    def retrieve_next_frame(self):
        self.c_reader.retrieveNextFrame()


    @property
    def has_next(self): return self.c_reader.hasNext()

    @property
    def has_next_frame(self): return self.c_reader.hasNextFrame()

    @property
    def n_frames(self): return self.c_reader.getNumberOfFrames()

    @property
    def run_number(self): return self.c_reader.getRunNumber()

    @property
    def det_id(self): return self.c_reader.getDetectorID()

    @property
    def frame_index(self): return self.c_reader.getFrameIndex()

    @property
    def utc_seconds(self): return self.c_reader.getUTCSeconds()

    @property
    def utc_nanoseconds(self): return self.c_reader.getUTCNanoseconds()

    # Frame
    @property
    def dom_id(self):
        return self.c_reader.getModuleID()

    @property
    def number_of_received_packets(self):
        return self.c_reader.getUDPNumberOfReceivedPackets()

    @property
    def max_sequence_number(self):
        return self.c_reader.getUDPMaximalSequenceNumber()

    @property
    def has_udp_trailer(self):
        return self.c_reader.hasUDPTrailer();

    @property
    def white_rabbit_status(self):
        return self.c_reader.testWhiteRabbitStatus();

    @property
    def high_rate_veto(self):
        return self.c_reader.testHighRateVeto();

    @property
    def fifo_status(self):
        return self.c_reader.testFIFOStatus();

    def get_rates(self, np.ndarray[double, ndim=1, mode="c"] rates not None):
        self.c_reader.getRates(&rates[0])

    def get_hrvs(self, np.ndarray[int, ndim=1, mode="c"] hrvs not None):
        self.c_reader.getHRVs(&hrvs[0])

    def get_fifos(self, np.ndarray[int, ndim=1, mode="c"] fifos not None):
        self.c_reader.getFIFOs(&fifos[0])



# distutils: language = c++
# distutils: sources = src/JDAQEventReader.cpp
# vim:set ts=4 sts=4 sw=4 et:
from libcpp cimport bool

import numpy as np
cimport numpy as np
cimport cython

np.import_array()


cdef extern from "JDAQEventReader.h" namespace "jppy":
    cdef cppclass JDAQEventReader:
        JDAQEventReader() except +
        JDAQEventReader(char* filename) except +
        void retrieveNextEvent()
        int getFrameIndex()
        int getRunNumber()
        int getDetectorID()
        int getTriggerCounter()
        int getTriggerMask()
        int getSize()
        unsigned int getOverlays()
        int getUTCSeconds()
        int getUTCNanoseconds()
        bool hasNext()
        int getNumberOfSnapshotHits()
        void getHits(int* channel_ids, int* dom_ids, int* times, int* tots,
                     int* triggereds)


cdef class PyJDAQEventReader:
    cdef JDAQEventReader c_reader

    def __cinit__(self, char* filename):
        self.c_reader = JDAQEventReader(filename)

    def retrieve_next_event(self):
        self.c_reader.retrieveNextEvent()

    @property
    def frame_index(self):
        return self.c_reader.getFrameIndex()

    @property
    def run_id(self):
        return self.c_reader.getRunNumber()

    @property
    def det_id(self):
        return self.c_reader.getDetectorID()

    @property
    def trigger_counter(self):
        return self.c_reader.getTriggerCounter()

    @property
    def trigger_mask(self):
        return self.c_reader.getTriggerMask()

    @property
    def size(self):
        return self.c_reader.getSize()

    @property
    def overlays(self):
        return self.c_reader.getOverlays()

    @property
    def utc_seconds(self):
        return self.c_reader.getUTCSeconds()

    @property
    def utc_nanoseconds(self):
        return self.c_reader.getUTCNanoseconds()

    @property
    def number_of_snapshot_hits(self):
        return self.c_reader.getNumberOfSnapshotHits()

    @property
    def has_next(self):
        return self.c_reader.hasNext()

    def get_hits(self,
                 np.ndarray[int, ndim=1, mode="c"] channel_ids not None,
                 np.ndarray[int, ndim=1, mode="c"] dom_ids  not None,
                 np.ndarray[int, ndim=1, mode="c"] times not None,
                 np.ndarray[int, ndim=1, mode="c"] tots not None,
                 np.ndarray[int, ndim=1, mode="c"] triggereds not None):
        self.c_reader.getHits(&channel_ids[0], &dom_ids[0], &times[0], &tots[0],
                              &triggereds[0])



from jppy import PyJDAQEventReader
import sys
import numpy as np


try:
    reader = PyJDAQEventReader(sys.argv[1])
except IndexError:
    raise SystemExit("Usage: {0} FILENAME".format(__file__))

for i in range(4):
    print(42*"=")
    print("Event #{0}".format(i))
    reader.retrieve_next_event()
    print("  Frame Index: {0}".format(reader.get_frame_index()))
    print("  Amount of Snapshot Hits: {0}".format(reader.get_number_of_snapshot_hits()))

    n = reader.get_number_of_snapshot_hits()
    channel_ids = np.zeros(n, dtype='i')
    dom_ids = np.zeros(n, dtype='i')
    times = np.zeros(n, dtype='i')
    tots = np.zeros(n, dtype='i')
    triggereds = np.zeros(n, dtype='i')
    reader.get_hits(channel_ids, dom_ids, times, tots, triggereds)
    print("  Snapshot Hits: \n"
          "    Channel IDs: {3}\n"
          "    DOM IDs:     {2}\n"
          "    Times:       {1}\n"
          "    ToTs:        {0}\n"
          "    Triggereds:  {4}"
          .format(tots, times, dom_ids, channel_ids, triggereds))




from jppy.daqtimeslicereader import PyJDAQTimesliceReader
import numpy as np

filename = 'test.root'

r = PyJDAQTimesliceReader(filename, "JDAQTimesliceL1")

r.retrieve_timeslice(0)
print(r.n_frames)
print(r.number_of_hits)

channel_ids = np.zeros(r.number_of_hits, dtype='i')
dom_ids = np.zeros(r.number_of_hits, dtype='i')
times = np.zeros(r.number_of_hits, dtype='i')
tots = np.zeros(r.number_of_hits, dtype='i')

r.get_hits(channel_ids, dom_ids, times, tots)
print(tots)

print(np.unique(dom_ids))



#!/usr/bin/env python

from jppy.daqsummaryslicereader import PyJDAQSummarysliceReader

r = PyJDAQSummarysliceReader("KM3NeT_00000013_00000501.root")

i = 0
while r.has_next:
    r.retrieve_next_summaryslice()
    print("Frame index: {0}".format(r.frame_index))
    print("Detector ID: {0}".format(r.det_id))
    print("Run Number:  {0}".format(r.run_number))
    print("UTC Seconds: {0}".format(r.utc_seconds))
    print("UTC NanoSeconds:  {0}".format(r.utc_nanoseconds))
    j = 0
    print("Number of Frames: {0}".format(r.number_of_frames))
    while r.has_next_frame:
        print(r.number_of_received_packets, r.max_sequence_number, r.module_id,
              r.has_udp_trailer, r.white_rabbit_status, r.high_rate_veto,
              r.fifo_status)
        r.retrieve_next_frame()
        j += 1
    i += 1





#!/usr/bin/env python
# coding=utf-8
# Filename: setup.py
"""
jppy setup script.

"""
from setuptools import setup
from distutils.extension import Extension
import os
import subprocess as sp

VERSION = '2.0'

try:
    from Cython.Build import cythonize
    import numpy
except Exception:
    raise SystemExit("Please install Cython and Numpy:\n\n"
                     "    pip install cython numpy")

try:
    os.environ['JPP_DIR']
except KeyError:
    raise SystemExit("\n$JPP_DIR is not set. "
                     "You need to activate the JPP environment!")

try:
    ROOT_INC = sp.Popen(["root-config", "--incdir"],
                        stdout=sp.PIPE).communicate()[0].strip().decode()
    ROOT_LIB = sp.Popen(["root-config", "--libdir"],
                        stdout=sp.PIPE).communicate()[0].strip().decode()
    JPP_INC = os.environ['JPP_DIR'] + "/software"
    JPP_LIB = os.environ['JPP_LIB']
    # AANET_INC = os.environ['AANET_INC']           # not defined in all envs!
    AANET_INC = os.environ['AADIR']
    AANET_INC_EVT = os.path.join(AANET_INC, 'evt')
    AANET_INC_UTIL = os.path.join(AANET_INC, 'util')
    AANET_LIB = os.environ['AANET_LIB']
except OSError:
    raise ImportError("Cannot detect JPP!")


# compile_args = ['-g', '-std=c++11']  # ROOT6
compile_args = []


extensions = [
    Extension("jppy/*", ["jppy/*.pyx"],
              include_dirs=['src/', JPP_INC, AANET_INC, AANET_INC_EVT,
                            AANET_INC_UTIL, ROOT_INC, numpy.get_include()],
              library_dirs=[ROOT_LIB, JPP_LIB, AANET_LIB],
              libraries=['KM3NeTDAQROOT', 'evtROOT', 'aa', 'jaanetROOT',
                         'triggerROOT',
                         'Cint',    # ROOT5
                         # 'Cling',  # ROOT6
                         'pthread', 'dl', 'util', 'm',
                         'Core', 'RIO', 'Net', 'Hist', 'Graf',
                         'Graf3d', 'Tree', 'Rint', 'Matrix', 'Physics',
                         'MathCore', 'Gpad', 'Thread'],
              extra_compile_args=compile_args,
              language='c++',
              # extra_link_args=[]
              ),
]

setup(name='jppy',
      version=VERSION,
      url='http://git.km3net.de/tgal/jppy.git',
      description='Python bindings for JPP',
      author='Tamas Gal',
      author_email='tgal@km3net.de',
      packages=['jppy'],
      setup_requires=['setuptools>=24.3', 'pip>=9.0.1', 'cython', 'numpy'],
      install_requires=['cython', 'numpy'],
      ext_modules=cythonize(extensions, language='c++', gdb_debug=True),
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Intended Audience :: Developers',
          'Intended Audience :: Science/Research',
          'Programming Language :: Python',
          ],
      )



# distutils: language = c++
# distutils: sources = src/JDAQTimesliceReader.cpp
# vim:set ts=4 sts=4 sw=4 et:
from libcpp cimport bool

import numpy as np
cimport numpy as np
cimport cython

np.import_array()


cdef extern from "JDAQTimesliceReader.h" namespace "jppy":
    cdef cppclass JDAQTimesliceReader:
        JDAQTimesliceReader() except +
        JDAQTimesliceReader(char* filename, char* stream) except +
        void initTreeScanner()
        void retrieveTimeslice(int index)
        void retrieveTimesliceAtFrameIndex(int frame_index)
        int getNumberOfTimeslices()
        int getNumberOfFrames()
        void getHits(int* channel_ids, int* dom_ids, int* times, int* tots)
        int getNumberOfHits()
        int getModuleID()
        int getFrameIndex()
        int getUTCSeconds()
        int getUTCNanoseconds()
        int getUDPNumberOfReceivedPackets()
        int getUDPMaximalSequenceNumber()
        bool hasUDPTrailer()
        bool testWhiteRabbitStatus()
        bool testHighRateVeto()
        bool testFIFOStatus()


cdef class PyJDAQTimesliceReader:
    cdef JDAQTimesliceReader c_reader
    cdef public object slices

    def __cinit__(self, char* filename, char* stream):
        self.c_reader = JDAQTimesliceReader(filename, stream)

    def init_tree_scanner(self):
        self.c_reader.initTreeScanner()

    def retrieve_timeslice(self, int index):
        self.c_reader.retrieveTimeslice(index)

    def retrieve_timeslice_at_frame_index(self, int frame_index):
        self.c_reader.retrieveTimesliceAtFrameIndex(frame_index)

    @property
    def n_timeslices(self):
        return self.c_reader.getNumberOfTimeslices()

    @property
    def n_frames(self):
        return self.c_reader.getNumberOfFrames()

    def get_hits(self,
                 np.ndarray[int, ndim=1, mode="c"] channel_ids not None,
                 np.ndarray[int, ndim=1, mode="c"] dom_ids  not None,
                 np.ndarray[int, ndim=1, mode="c"] times not None,
                 np.ndarray[int, ndim=1, mode="c"] tots not None):
        self.c_reader.getHits(&channel_ids[0],
                              &dom_ids[0],
                              &times[0],
                              &tots[0])

    # Frame
    @property
    def frame_index(self):
        return self.c_reader.getFrameIndex()

    @property
    def dom_id(self):
        return self.c_reader.getModuleID()

    @property
    def utc_seconds(self): return self.c_reader.getUTCSeconds()

    @property
    def utc_nanoseconds(self): return self.c_reader.getUTCNanoseconds()

    @property
    def number_of_received_packets(self):
        return self.c_reader.getUDPNumberOfReceivedPackets()

    @property
    def max_sequence_number(self):
        return self.c_reader.getUDPMaximalSequenceNumber()

    @property
    def has_udp_trailer(self):
        return self.c_reader.hasUDPTrailer();

    @property
    def white_rabbit_status(self):
        return self.c_reader.testWhiteRabbitStatus();

    @property
    def high_rate_veto(self):
        return self.c_reader.testHighRateVeto();

    @property
    def fifo_status(self):
        return self.c_reader.testFIFOStatus();

    @property
    def number_of_hits(self):
        return self.c_reader.getNumberOfHits()



# distutils: language=c++
# distutils: sources = src/JFitReader.cpp
# vim:set ts=4 sts=4 sw=4 et:
from libcpp cimport bool

import numpy as np
cimport numpy as np
cimport cython

np.import_array()

# cdef extern from "JFit/JFitParameters.hh" namespace "jppy":
#     cdef enum JFitParameter:
#         JGANDALF_BETA0_RAD = 0
#         JGANDALF_BETA1_RAD = 1
#         JGANDALF_CHI2 = 2
#         JGANDALF_NUMBER_OF_HITS = 3
#         JENERGY_ENERGY = 4
#         JENERGY_CHI2 = 5
#         JGANDALF_LAMBDA = 6
#         JGANDALF_NUMBER_OF_ITERATIONS = 7
#         JSTART_NPE_MIP = 8
#         JSTART_NPE_MIP_TOTAL = 9
#         JSTART_LENGTH_METRES = 10
#         JVETO_NPE = 11
#         JVETO_NUMBER_OF_HITS = 12

cdef extern from "JFitReader.h" namespace "jppy":
    cdef cppclass JFitReader:
        JFitReader() except +
        JFitReader(char* filename) except +
        void retrieveNextEvent()
        int getNFits()
        bool hasNext()
        void getFits(
            double* pos_xs,
            double* pos_ys,
            double* pos_zs,
            double* dir_xs,
            double* dir_ys,
            double* dir_zs,
            int* ndfs,
            double* times,
            double* qualities,
            double* energies
        )


cdef class PyJFitReader:
    cdef JFitReader c_reader

    def __cinit__(self, char* filename):
        self.c_reader = JFitReader(filename)

    def retrieve_next_event(self):
        self.c_reader.retrieveNextEvent()

    @property
    def n_fits(self):
        return self.c_reader.getNFits()

    @property
    def has_next(self):
        return self.c_reader.hasNext()

    def get_fits(self,
                 np.ndarray[double, ndim=1, mode="c"] pos_xs not None,
                 np.ndarray[double, ndim=1, mode="c"] pos_ys not None,
                 np.ndarray[double, ndim=1, mode="c"] pos_zs not None,
                 np.ndarray[double, ndim=1, mode="c"] dir_xs not None,
                 np.ndarray[double, ndim=1, mode="c"] dir_ys not None,
                 np.ndarray[double, ndim=1, mode="c"] dir_zs not None,
                 np.ndarray[int, ndim=1, mode="c"] ndfs not None,
                 np.ndarray[double, ndim=1, mode="c"] times not None,
                 np.ndarray[double, ndim=1, mode="c"] qualities not None,
                 np.ndarray[double, ndim=1, mode="c"] energies not None,
                 ):
        self.c_reader.getFits(
                &pos_xs[0],
                &pos_ys[0],
                &pos_zs[0],
                &dir_xs[0],
                &dir_ys[0],
                &dir_zs[0],
                &ndfs[0],
                &times[0],
                &qualities[0],
                &energies[0],
                )



#!/usr/bin/env python
"""
Test the fitreader. This Plots the Z-dir of the first JFit given by JEvt.
By Giuliano + Moritz.

Usage:
    test_fitreader.py [-p PLOTFILE] INFILE

Options:
    -p PLOTFILE         Name of the pdf file [default: test_fitreader.pdf]
"""
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd


def read_fits(fname):
    from jppy.fitreader import PyJFitReader

    fr = PyJFitReader(fname)
    dirs_z = []

    while fr.has_next:
        fr.retrieve_next_event()
        n_fits = fr.n_fits

        if n_fits == 0:
            continue

        buf_size = 500000
        pos_xs = np.zeros(buf_size, dtype='d')
        pos_ys = np.zeros(buf_size, dtype='d')
        pos_zs = np.zeros(buf_size, dtype='d')
        dir_xs = np.zeros(buf_size, dtype='d')
        dir_ys = np.zeros(buf_size, dtype='d')
        dir_zs = np.zeros(buf_size, dtype='d')
        ndfs = np.zeros(buf_size, dtype='i')
        times = np.zeros(buf_size, dtype='d')
        qualities = np.zeros(buf_size, dtype='d')
        energies = np.zeros(buf_size, dtype='d')

        fr.get_fits(
            pos_xs,
            pos_ys,
            pos_zs,
            dir_xs,
            dir_ys,
            dir_zs,
            ndfs,
            times,
            qualities,
            energies,
        )

        dirs_z.append(dir_zs[0])

    return dirs_z


if __name__ == '__main__':
    from docopt import docopt
    args = docopt(__doc__)
    infile = args['INFILE']
    plotfile = args['-p']

    dirs_z = read_fits(infile)
    dirs_z = pd.Series(dirs_z)

    entries = len(dirs_z)
    mean = sum(dirs_z) / entries

    plt.clf()
    plt.hist(dirs_z, bins=50, range=[-1., 1.], linewidth=2, histtype='step',
             label=' Entries ' + str(entries) + '\n Mean: ' + str(mean)[0:5])
    plt.figtext(1.0, 0.2, dirs_z.describe())
    plt.xlabel('Z-Dir of first JFit in JEvt', size=12)
    plt.grid()
    plt.legend(bbox_to_anchor=(0.3, 0.9), ncol=1, loc='best')
    # plt.show()
    plt.savefig(plotfile)
    plt.close()



import km3pipe as kp

def printer(blob):
    print(blob)
    return blob

pipe = kp.Pipeline()
pipe.attach(kp.io.jpp.TimeslicePump, filename="test.root", stream="SN")
pipe.attach(printer)
pipe.drain(10)
